/*============================================================================
  File:     00_QSPrep.sql

  SQL Server Versions: 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Restore database
	*you may need to change the backup
	and restore locations
*/
USE [master];
GO
RESTORE DATABASE [WideWorldImporters] 
FROM  DISK = N'C:\Backups\WideWorldImportersEnlarged.bak' 
WITH  FILE = 1,  
MOVE N'WWI_Primary' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters.mdf',  
MOVE N'WWI_UserData' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters_UserData.ndf',  
MOVE N'WWI_Log' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters.ldf',  
MOVE N'WWI_InMemory_Data_1' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters_InMemory_Data_1',  
NOUNLOAD, 
REPLACE, 
STATS = 5;
GO


/*
	Enable query store and clear anything that may be in there

*/
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE = ON;
GO
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE, 
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30), 
	DATA_FLUSH_INTERVAL_SECONDS = 300,  
	INTERVAL_LENGTH_MINUTES = 10, 
	MAX_STORAGE_SIZE_MB = 512, 
	QUERY_CAPTURE_MODE = ALL, 
	SIZE_BASED_CLEANUP_MODE = AUTO, 
	MAX_PLANS_PER_QUERY = 200)
GO


/*
	Create stored procedures for testing
*/
USE [WideWorldImporters]
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_CustomerTransactionInfo];
GO

CREATE PROCEDURE [Sales].[usp_CustomerTransactionInfo]
	@CustomerID INT
AS	

	SELECT [CustomerID], SUM([AmountExcludingTax])
	FROM [Sales].[CustomerTransactions]
	WHERE [CustomerID] = @CustomerID
	GROUP BY [CustomerID];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_GetPersonInfo];
GO

CREATE PROCEDURE [Application].[usp_GetPersonInfo] (@PersonID INT)
AS

	SELECT 
		[p].[FullName], 
		[p].[EmailAddress], 
		[c].[FormalName]
	FROM [Application].[People] [p]
	LEFT OUTER JOIN [Application].[Countries] [c] 
		ON [p].[PersonID] = [c].[LastEditedBy]
	WHERE [p].[PersonID] = @PersonID;
GO



DROP PROCEDURE IF EXISTS [Sales].[usp_GetFullProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]
	@StockItemID INT
AS	

	SELECT 
		[o].[CustomerID], 
		[o].[OrderDate], 
		[ol].[StockItemID], 
		[ol].[Quantity],
		[ol].[UnitPrice]
	FROM [Sales].[Orders] [o]
	JOIN [Sales].[OrderLines] [ol] 
		ON [o].[OrderID] = [ol].[OrderID]
	WHERE [ol].[StockItemID] = @StockItemID
	ORDER BY [o].[OrderDate] DESC;

	SELECT
		[o].[CustomerID], 
		SUM([ol].[Quantity]*[ol].[UnitPrice])
	FROM [Sales].[Orders] [o]
	JOIN [Sales].[OrderLines] [ol] 
		ON [o].[OrderID] = [ol].[OrderID]
	WHERE [ol].[StockItemID] = @StockItemID
	GROUP BY [o].[CustomerID]
	ORDER BY [o].[CustomerID] ASC;
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_FindOrderByDescription];
GO


CREATE PROCEDURE [Sales].[usp_FindOrderByDescription]
	@Description VARCHAR(100)
AS	

	SELECT [ol].[StockItemID], [ol].[Description], [ol].[UnitPrice],
		[o].[CustomerID], [o].[SalespersonPersonID]
	FROM [Sales].[OrderLines] [ol]
	JOIN [Sales].[Orders] [o]
		ON [ol].[OrderID] = [o].[OrderID]
	WHERE [ol].[Description] = @Description 

GO

DROP PROCEDURE IF EXISTS [Sales].[usp_CustomerOrdersByDate];
GO


CREATE PROCEDURE [Sales].[usp_CustomerOrdersByDate]
	@StartDate DATE, @EndDate DATE
AS	

	SELECT [o].[CustomerID], [ol].[StockItemID], SUM([ol].[Quantity])
	FROM Sales.Orders [o]
	JOIN Sales.OrderLines [ol]
		ON [o].[OrderID] = [ol].[OrderID]
	WHERE [o].[OrderDate] BETWEEN @StartDate AND @EndDate
	GROUP BY [o].[CustomerID], [ol].[StockItemID]
	ORDER BY [o].[CustomerID], [ol].[StockItemID]

GO

UPDATE STATISTICS Sales.Orders WITH NORECOMPUTE;
GO
UPDATE STATISTICS Sales.OrderLines WITH NORECOMPUTE;
GO

CREATE NONCLUSTERED INDEX NCI_Orders_OrderDate
	ON Sales.Orders (OrderDate)
GO

DECLARE @StartDate1 DATE = '2016-01-01'
DECLARE @EndDate1 DATE = '2016-01-31'
EXEC [Sales].[usp_CustomerOrdersByDate] @StartDate1, @EndDate1
GO

DECLARE @StartDate2 DATE = '2016-01-01'
DECLARE @EndDate2 DATE = '2016-01-31'
EXEC [Sales].[usp_CustomerOrdersByDate] @StartDate2, @EndDate2
GO

DECLARE @StartDate3 DATE = '2016-01-01'
DECLARE @EndDate3 DATE = '2016-01-31'
EXEC [Sales].[usp_CustomerOrdersByDate] @StartDate3, @EndDate3
GO
/*
	Create indexes to support for
	Invoice/Invoice Lines
	and PurchaseOrder/PurchaseOrderLines
*/


CREATE NONCLUSTERED INDEX [NCI_InvoiceLines_PS] 
	ON [Sales].[InvoiceLines] ([Description])
	INCLUDE ([InvoiceLineID], [InvoiceID], [UnitPrice])
ON [USERDATA];
GO

CREATE NONCLUSTERED INDEX [NCI_Invoices_PS] 
	ON [Sales].[Invoices] ([InvoiceID])
	INCLUDE ([CustomerID], [SalespersonPersonID])
ON [USERDATA];
GO

CREATE NONCLUSTERED INDEX [NCI_PurchaseOrderLines_PS]
	ON [Purchasing].[PurchaseOrderLines] ([Description])
	INCLUDE ([PurchaseOrderID], [PurchaseOrderLineID], [LastEditedWhen])
ON [USERDATA];
GO

CREATE NONCLUSTERED INDEX [NCI_PurchaseOrders_PS]
	ON [Purchasing].[PurchaseOrders] ([PurchaseOrderID])
	INCLUDE ([OrderDate], [ExpectedDeliveryDate])
ON [USERDATA];
GO


CREATE NONCLUSTERED INDEX [NCI_OrderLines_StockItemID]
	ON [Sales].[OrderLines] ([StockItemID])
	INCLUDE ([OrderID],[Quantity],[UnitPrice])
ON [USERDATA];
GO

/*
	clear QS before start
*/
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE CLEAR;
GO

