/*============================================================================
  File:     01_WorstPerformers.sql

  SQL Server Versions: 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Check resource use through the UI
*/

/*
	Find top resource consumers with TSQL
	Longest execution times in the last hour
*/

USE [WideWorldImporters];
GO

SELECT 
	TOP 10 AVG([rs].[avg_duration]) [AvgDuration], 
	SUM([rs].[count_executions]) [TotalExecutions],
	[qsq].[query_id],  
	[qst].[query_sql_text], 
	CASE
		WHEN [qsq].[object_id] = 0 THEN N'Ad-hoc'
		ELSE OBJECT_NAME([qsq].[object_id]) 
	END AS [ObjectName],
	[qsp].[plan_id], 
	TRY_CONVERT(XML, [qsp].[query_plan]) [QueryPlan]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id] 
WHERE [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
AND [rs].[execution_type] = 0
GROUP BY [qsq].[query_id], [qst].[query_sql_text], 
[qsq].[object_id], [qsp].[plan_id], [qsp].[query_plan]
ORDER BY AVG([rs].[avg_duration]) DESC;  
GO


/*
	Highest logical IO in last 1 hour
*/
SELECT 
	TOP 10 AVG([rs].[avg_logical_io_reads]) [AvgLogicalIO], 
	SUM([rs].[count_executions]) [TotalExecutions],
	[qsq].[query_id],  
	[qst].[query_sql_text], 
	CASE
		WHEN [qsq].[object_id] = 0 THEN N'Ad-hoc'
		ELSE OBJECT_NAME([qsq].[object_id]) 
	END AS [ObjectName],
	[qsp].[plan_id], 
	TRY_CONVERT(XML, [qsp].[query_plan]) [QueryPlan]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id] 
WHERE [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
GROUP BY [qsq].[query_id], [qst].[query_sql_text], 
[qsq].[object_id], [qsp].[plan_id], [qsp].[query_plan]
ORDER BY AVG([rs].[avg_logical_io_reads]) DESC;  
GO


/*
	Highest execution count in last 1 hour
*/
SELECT 
	TOP 10 SUM([rs].[count_executions]) [TotalExecutions],
	[qsq].[query_id],  
	[qst].[query_sql_text], 
	CASE
		WHEN [qsq].[object_id] = 0 THEN N'Ad-hoc'
		ELSE OBJECT_NAME([qsq].[object_id]) 
	END AS [ObjectName],
	[qsp].[plan_id], 
	TRY_CONVERT(XML, [qsp].[query_plan]) [QueryPlan]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id] 
WHERE [rs].[last_execution_time] > DATEADD(HOUR, -1, GETUTCDATE())  
AND [rs].[execution_type] = 0
GROUP BY [qsq].[query_id], [qst].[query_sql_text], 
[qsq].[object_id], [qsp].[plan_id], [qsp].[query_plan]
ORDER BY SUM([rs].[count_executions]) DESC;  
GO


/*
	Why does the same SP have multiple rows in the above query?
*/
SELECT 	
	[qsq].[query_id],  
	[qst].[query_sql_text], 
	[qsq].[object_id], 
	OBJECT_NAME([qsq].[object_id]) 
FROM  [sys].[query_store_query] [qsq]
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
WHERE [qsq].[object_id] = OBJECT_ID(N'Sales.usp_GetFullProductInfo');
GO

SELECT OBJECT_DEFINITION(OBJECT_ID(N'Sales.usp_GetFullProductInfo'));
GO

/*
	Stored procedure definition
*/
CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]   @StockItemID INT  
AS      
SELECT     
[o].[CustomerID],     
[o].[OrderDate],     
[ol].[StockItemID],     
[ol].[Quantity],    
[ol].[UnitPrice]   
FROM [Sales].[Orders] [o]   JOIN [Sales].[OrderLines] [ol]     
ON [o].[OrderID] = [ol].[OrderID]   
WHERE [ol].[StockItemID] = @StockItemID   
ORDER BY [o].[OrderDate] DESC;     
  
SELECT    
[o].[CustomerID],     
SUM([ol].[Quantity]*[ol].[UnitPrice])   
FROM [Sales].[Orders] [o]   JOIN [Sales].[OrderLines] [ol]     
ON [o].[OrderID] = [ol].[OrderID]   
WHERE [ol].[StockItemID] = @StockItemID   
GROUP BY [o].[CustomerID]   
ORDER BY [o].[CustomerID] ASC;  